@@ -6,6 +6,7 @@
 
 #include "applicationcontroller.h"
 #include "documentmanager.h"
+#include "downloadjob.h"
 #include "favoritelocationmodel.h"
 #include "filehelper.h"
 #include "genericpkpass.h"
@@ -22,14 +23,16 @@
 #include "tripgroupmanager.h"
 #include <itinerary_version_detailed.h>
 
-#include <kitinerary_version.h>
 #include <KItinerary/CreativeWork>
 #include <KItinerary/DocumentUtil>
 #include <KItinerary/ExtractorCapabilities>
 #include <KItinerary/ExtractorEngine>
 #include <KItinerary/ExtractorDocumentNode>
+#include <KItinerary/ExtractorPostprocessor>
 #include <KItinerary/File>
 #include <KItinerary/JsonLdDocument>
+#include <KItinerary/Place>
+#include <KItinerary/Reservation>
 
 #include <KPkPass/Pass>
 
@@ -176,9 +179,9 @@ void ApplicationController::setPassManager(PassManager *passMgr)
     m_passMgr = passMgr;
 }
 
-void ApplicationController::setHealthCertificateManager(HealthCertificateManager *healthCertMgr)
+bool ApplicationController::probablyUrl(const QString &text)
 {
-    m_healthCertMgr = healthCertMgr;
+    return (text.startsWith(QLatin1String("https://")) || text.startsWith(QLatin1String("http://"))) && text.size() < 256;
 }
 
 #ifdef Q_OS_ANDROID
@@ -204,7 +207,7 @@ void ApplicationController::importFromIntent(const KAndroidExtras::Intent &inten
         return;
     }
 
-    // shared data, e.g. from email applications like FairMail
+    // shared content, e.g. URL from the browser or email applications like FairMail
     if (action == Intent::ACTION_SEND || action == Intent::ACTION_SEND_MULTIPLE) {
         const QString type = intent.getType();
         const auto subject = intent.getStringExtra(Intent::EXTRA_SUBJECT);
@@ -214,6 +217,11 @@ void ApplicationController::importFromIntent(const KAndroidExtras::Intent &inten
         const QStringList attachments = ItineraryActivity().attachmentsForIntent(intent);
         qCInfo(Log) << attachments;
 
+        if (probablyUrl(text)) {
+            importFromUrl(QUrl(text));
+            return;
+        }
+
         KMime::Message msg;
         msg.subject()->fromUnicodeString(subject, "utf-8");
         for (const auto &f : from) {
@@ -270,9 +278,10 @@ void ApplicationController::importFromClipboard()
     else if (md->hasText()) {
         const auto content = md->data(QLatin1String("text/plain"));
 
+        const QString contentString = QString::fromUtf8(content);
         // URL copied as plain text
-        if (content.startsWith("https://") && content.size() < 256) {
-            const QUrl url(QString::fromUtf8(content));
+        if (probablyUrl(contentString)) {
+            const QUrl url(contentString);
             if (url.isValid()) {
                 importFromUrl(url);
                 return;
@@ -302,18 +311,14 @@ void ApplicationController::importFromUrl(const QUrl &url)
         if (!m_nam ) {
             m_nam = new QNetworkAccessManager(this);
         }
-        auto reqUrl(url);
-        reqUrl.setScheme(QLatin1String("https"));
-        QNetworkRequest req(reqUrl);
-        req.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
-        auto reply = m_nam->get(req);
-        connect(reply, &QNetworkReply::finished, this, [this, reply]() {
-            if (reply->error() != QNetworkReply::NoError) {
-                qCDebug(Log) << reply->url() << reply->errorString();
-                Q_EMIT infoMessage(i18n("Download failed: %1", reply->errorString()));
+        auto job = new DownloadJob(url, m_nam, this);
+        connect(job, &DownloadJob::finished, this, [this, job]() {
+            job->deleteLater();
+            if (job->hasError()) {
+                Q_EMIT infoMessage(job->errorMessage());
                 return;
             }
-            importData(reply->readAll());
+            importData(job->data());
         });
         return;
     }
@@ -361,6 +366,23 @@ void ApplicationController::importLocalFile(const QUrl &url)
     importData(f.readAll(), fileName);
 }
 
+QString ApplicationController::addAttachableDocument(const QString &fileName, const QByteArray &data)
+{
+    // check if there is a document we want to attach here
+    QMimeDatabase db;
+    const auto mt = db.mimeTypeForFileNameAndData(fileName, data);
+    if (mt.name() == QLatin1String("application/pdf") || mt.name() == QLatin1String("message/rfc822") || mt.name() == QLatin1String("application/mbox")) {
+        DigitalDocument docInfo;
+        docInfo.setName(fileName);
+        docInfo.setEncodingFormat(mt.name());
+        const auto docId = DocumentUtil::idForContent(data);
+        m_docMgr->addDocument(docId, docInfo, data);
+        return docId;
+    }
+
+    return {};
+}
+
 bool ApplicationController::importData(const QByteArray &data, const QString &fileName)
 {
     qCDebug(Log);
@@ -380,24 +402,15 @@ bool ApplicationController::importData(const QByteArray &data, const QString &fi
     // user opened the file, so we can be reasonably sure they assume it contains
     // relevant content, so try expensive extraction methods too
     engine.setHints(ExtractorEngine::ExtractFullPageRasterImages);
-#if KITINERARY_VERSION >= QT_VERSION_CHECK(5, 22, 41)
     engine.setHints(engine.hints() | ExtractorEngine::ExtractGenericIcalEvents);
-#endif
     engine.setContextDate(QDateTime(QDate::currentDate(), QTime(0, 0)));
     engine.setData(data, fileName);
     const auto extractorResult = JsonLdDocument::fromJson(engine.extract());
     const auto resIds = m_resMgr->importReservations(extractorResult);
     if (!resIds.isEmpty()) {
         // check if there is a document we want to attach here
-        QMimeDatabase db;
-        const auto mt = db.mimeTypeForFileNameAndData(fileName, data);
-        if (mt.name() == QLatin1String("application/pdf")) { // TODO support more file types (however we certainly want to exclude pkpass and json here)
-            DigitalDocument docInfo;
-            docInfo.setName(fileName);
-            docInfo.setEncodingFormat(mt.name());
-            const auto docId = DocumentUtil::idForContent(data);
-            m_docMgr->addDocument(docId, docInfo, data);
-
+        const auto docId = addAttachableDocument(fileName, data);
+        if (!docId.isEmpty()) {
             for (const auto &resId : resIds) {
                 auto res = m_resMgr->reservation(resId);
                 if (DocumentUtil::addDocumentId(res, docId)) {
@@ -416,17 +429,59 @@ bool ApplicationController::importData(const QByteArray &data, const QString &fi
 
     // look for health certificate barcodes instead
     // if we don't find anything, try to import as health certificate directly
-    if (importHealthCertificateRecursive(engine.rootDocumentNode()) || m_healthCertMgr->importCertificate(data)) {
+    if (importHealthCertificateRecursive(engine.rootDocumentNode()) || healthCertificateManager()->importCertificate(data)) {
         Q_EMIT infoMessage(i18n("Health certificate imported."));
         healthCertImported = true;
     }
 
     // look for time-less passes/program memberships/etc
-    if (m_passMgr->import(extractorResult) || (resIds.isEmpty() && !healthCertImported && importGenericPkPass(engine.rootDocumentNode()))) {
+    if (const auto passIds = m_passMgr->import(extractorResult); !passIds.isEmpty()) {
+        const auto docId = addAttachableDocument(fileName, data);
+        if (!docId.isEmpty()) {
+            for (const auto &passId : passIds) {
+                auto pass = m_passMgr->pass(passId);
+                if (DocumentUtil::addDocumentId(pass, docId)) {
+                    m_passMgr->update(passId, pass);
+                }
+            }
+        }
+        Q_EMIT infoMessage(i18np("One pass imported.", "%1 passes imported.", passIds.size()));
+        success = true;
+    } else if (resIds.isEmpty() && !healthCertImported && importGenericPkPass(engine.rootDocumentNode())) {
         Q_EMIT infoMessage(i18n("Pass imported."));
         success = true;
     }
 
+    // check for things we can add reservations for manually
+    if (!success && !extractorResult.isEmpty()) {
+        ExtractorPostprocessor postProc;
+        postProc.setValidationEnabled(false);
+        postProc.process(extractorResult);
+        const auto postProcssedResult = postProc.result();
+        ExtractorValidator validator;
+        validator.setAcceptedTypes<LodgingBusiness, FoodEstablishment, LocalBusiness>();
+        validator.setAcceptOnlyCompleteElements(true);
+        for (const auto &resFor : postProcssedResult) {
+            if (!validator.isValidElement(resFor)) {
+                continue;
+            }
+            if (JsonLd::isA<LodgingBusiness>(resFor)) {
+                LodgingReservation res;
+                res.setReservationFor(resFor);
+                Q_EMIT editNewHotelReservation(res);
+                success = true;
+                break;
+            } else if (JsonLd::isA<FoodEstablishment>(resFor) || JsonLd::isA<LocalBusiness>(resFor)) {
+                // LocalBusiness is frequently used for restaurants in website annotations
+                FoodEstablishmentReservation res;
+                res.setReservationFor(resFor);
+                Q_EMIT editNewRestaurantReservation(res);
+                success = true;
+                break;
+            }
+        }
+    }
+
     // nothing found
     if (!success && !healthCertImported) {
         Q_EMIT infoMessage(i18n("Nothing imported."));
@@ -456,7 +511,7 @@ bool ApplicationController::importGenericPkPass(const KItinerary::ExtractorDocum
 {
     if (node.mimeType() == QLatin1String("application/vnd.apple.pkpass")) {
         const auto pass = node.content<KPkPass::Pass*>();
-        if (pass->type() == KPkPass::Pass::Coupon || pass->type() == KPkPass::Pass::StoreCard) {
+        if (!pass || pass->type() == KPkPass::Pass::Coupon || pass->type() == KPkPass::Pass::StoreCard) {
             // no support for displaying those yet
             return false;
         }
@@ -509,7 +564,7 @@ void ApplicationController::exportToFile(const QUrl &url)
     exporter.exportFavoriteLocations(m_favLocModel);
     exporter.exportTransfers(m_resMgr, m_transferMgr);
     exporter.exportPasses(m_passMgr);
-    exporter.exportHealthCertificates(m_healthCertMgr);
+    exporter.exportHealthCertificates(healthCertificateManager());
     exporter.exportLiveData();
     exporter.exportSettings();
     Q_EMIT infoMessage(i18n("Export completed."));
@@ -579,7 +634,7 @@ bool ApplicationController::importBundle(KItinerary::File *file)
         count += importer.importFavoriteLocations(m_favLocModel);
         count += importer.importTransfers(m_resMgr, m_transferMgr);
         count += importer.importPasses(m_passMgr);
-        count += importer.importHealthCertificates(m_healthCertMgr);
+        count += importer.importHealthCertificates(healthCertificateManager());
         count += importer.importLiveData(m_liveDataMgr);
         count += importer.importSettings();
     }
@@ -595,10 +650,10 @@ bool ApplicationController::importHealthCertificateRecursive(const ExtractorDocu
     if (node.childNodes().size() == 1 && (node.mimeType() == QLatin1String("internal/qimage") || node.mimeType() == QLatin1String("application/vnd.apple.pkpass"))) {
         const auto &child = node.childNodes()[0];
         if (child.isA<QString>()) {
-            return m_healthCertMgr->importCertificate(child.content<QString>().toUtf8());
+            return healthCertificateManager()->importCertificate(child.content<QString>().toUtf8());
         }
         if (child.isA<QByteArray>()) {
-            return m_healthCertMgr->importCertificate(child.content<QByteArray>());
+            return healthCertificateManager()->importCertificate(child.content<QByteArray>());
         }
     }
 
@@ -636,13 +691,13 @@ void ApplicationController::importMimeMessage(KMime::Message *msg)
     }
 }
 
-void ApplicationController::addDocument(const QString &batchId, const QUrl &url)
+QString ApplicationController::addDocumentFromFile(const QUrl &url)
 {
     if (!url.isValid()) {
-        return;
+        return {};
     }
 
-    const auto docId = QUuid::createUuid().toString();
+    const auto docId = QUuid::createUuid().toString(QUuid::WithoutBraces);
 
     DigitalDocument docInfo;
 #ifdef Q_OS_ANDROID
@@ -655,7 +710,15 @@ void ApplicationController::addDocument(const QString &batchId, const QUrl &url)
 #endif
 
     m_docMgr->addDocument(docId, docInfo, FileHelper::toLocalFile(url));
+    return docId;
+}
 
+void ApplicationController::addDocumentToReservation(const QString &batchId, const QUrl &url)
+{
+    const auto docId = addDocumentFromFile(url);
+    if (docId.isEmpty()) {
+        return;
+    }
     const auto resIds = m_resMgr->reservationsForBatch(batchId);
     for (const auto &resId : resIds) {
         auto res = m_resMgr->reservation(resId);
@@ -664,7 +727,19 @@ void ApplicationController::addDocument(const QString &batchId, const QUrl &url)
     }
 }
 
-void ApplicationController::removeDocument(const QString &batchId, const QString &docId)
+void ApplicationController::addDocumentToPass(const QString &passId, const QUrl &url)
+{
+    const auto docId = addDocumentFromFile(url);
+    if (docId.isEmpty()) {
+        return;
+    }
+    auto pass = m_passMgr->pass(passId);
+    if (DocumentUtil::addDocumentId(pass, docId)) {
+        m_passMgr->update(passId, pass);
+    }
+}
+
+void ApplicationController::removeDocumentFromReservation(const QString &batchId, const QString &docId)
 {
     const auto resIds = m_resMgr->reservationsForBatch(batchId);
     for (const auto &resId : resIds) {
@@ -676,6 +751,15 @@ void ApplicationController::removeDocument(const QString &batchId, const QString
     m_docMgr->removeDocument(docId);
 }
 
+void ApplicationController::removeDocumentFromPass(const QString &passId, const QString &docId)
+{
+    auto pass = m_passMgr->pass(passId);
+    if (DocumentUtil::removeDocumentId(pass, docId)) {
+        m_passMgr->update(passId, pass);
+    }
+    m_docMgr->removeDocument(docId);
+}
+
 void ApplicationController::openDocument(const QUrl &url)
 {
 #ifdef Q_OS_ANDROID
@@ -712,4 +796,17 @@ QString ApplicationController::userAgent() const
     return QLatin1String("org.kde.itinerary/") + QCoreApplication::applicationVersion() + QLatin1String(" (kde-pim@kde.org)");
 }
 
+bool ApplicationController::hasHealthCertificateSupport() const
+{
+    return HealthCertificateManager::isAvailable();
+}
+
+HealthCertificateManager* ApplicationController::healthCertificateManager() const
+{
+    if (!m_healthCertMgr) {
+        m_healthCertMgr = new HealthCertificateManager(const_cast<ApplicationController*>(this));
+    }
+    return m_healthCertMgr;
+}
+
 #include "moc_applicationcontroller.cpp"
