@@ -40,6 +40,8 @@ class ApplicationController : public QObject
     Q_PROPERTY(QString extractorCapabilities READ extractorCapabilities CONSTANT)
     Q_PROPERTY(QVariant aboutData READ aboutData CONSTANT)
     Q_PROPERTY(QString userAgent READ userAgent CONSTANT)
+    Q_PROPERTY(bool hasHealthCertificateSupport READ hasHealthCertificateSupport CONSTANT)
+    Q_PROPERTY(HealthCertificateManager *healthCertificateManager READ healthCertificateManager CONSTANT)
 public:
     explicit ApplicationController(QObject *parent = nullptr);
     ~ApplicationController() override;
@@ -54,7 +56,6 @@ public:
     void setLiveDataManager(LiveDataManager *liveDataMgr);
     void setTripGroupManager(TripGroupManager *tripGroupMgr);
     void setPassManager(PassManager *passMgr);
-    void setHealthCertificateManager(HealthCertificateManager *healthCertMgr);
 
     // data import
     void importFromIntent(const KAndroidExtras::Intent &intent);
@@ -70,8 +71,10 @@ public:
     Q_INVOKABLE void exportTripToGpx(const QString &tripGroupId, const QUrl &url);
 
     // document attaching
-    Q_INVOKABLE void addDocument(const QString &batchId, const QUrl &url);
-    Q_INVOKABLE void removeDocument(const QString &batchId, const QString &docId);
+    Q_INVOKABLE void addDocumentToReservation(const QString &batchId, const QUrl &url);
+    Q_INVOKABLE void removeDocumentFromReservation(const QString &batchId, const QString &docId);
+    Q_INVOKABLE void addDocumentToPass(const QString &passId, const QUrl &url);
+    Q_INVOKABLE void removeDocumentFromPass(const QString &passId, const QString &docId);
     Q_INVOKABLE void openDocument(const QUrl &url);
 
     // about information
@@ -80,6 +83,10 @@ public:
     QVariant aboutData() const;
     QString userAgent() const;
 
+    // health certificate manager
+    bool hasHealthCertificateSupport() const;
+    HealthCertificateManager *healthCertificateManager() const;
+
 Q_SIGNALS:
     void clipboardContentChanged();
 
@@ -88,9 +95,14 @@ Q_SIGNALS:
 
     void openPageRequested(const QString &page);
 
+    /** Edit and add a new reservation. */
+    void editNewHotelReservation(const QVariant &res);
+    void editNewRestaurantReservation(const QVariant &res);
+
 private:
     bool hasClipboardContent() const;
 
+    static bool probablyUrl(const QString &text);
     void importLocalFile(const QUrl &url);
     bool importBundle(const QUrl &url);
     bool importBundle(const QByteArray &data);
@@ -101,6 +113,9 @@ private:
     void importNode(const KItinerary::ExtractorDocumentNode &node);
     bool importGenericPkPass(const KItinerary::ExtractorDocumentNode &node);
 
+    QString addDocumentFromFile(const QUrl &url);
+    QString addAttachableDocument(const QString &fileName, const QByteArray &data);
+
     static ApplicationController *s_instance;
 
     ReservationManager *m_resMgr = nullptr;
@@ -111,7 +126,7 @@ private:
     LiveDataManager *m_liveDataMgr = nullptr;
     TripGroupManager *m_tripGroupMgr = nullptr;
     PassManager *m_passMgr = nullptr;
-    HealthCertificateManager *m_healthCertMgr = nullptr;
+    mutable HealthCertificateManager *m_healthCertMgr = nullptr;
     QNetworkAccessManager *m_nam = nullptr;
 
     bool m_importLock = false;
