@@ -11,8 +11,10 @@
 #include "reservationhelper.h"
 #include "reservationmanager.h"
 #include "publictransport.h"
+#include "publictransportmatcher.h"
 
 #include <KItinerary/BusTrip>
+#include <KItinerary/Flight>
 #include <KItinerary/LocationUtil>
 #include <KItinerary/Place>
 #include <KItinerary/Reservation>
@@ -145,38 +147,6 @@ void LiveDataManager::checkForUpdates()
     pollForUpdates(true);
 }
 
-static bool isSameLine(const KPublicTransport::Line &lhs, const QString &trainName, const QString &trainNumber)
-{
-    KPublicTransport::Line rhs;
-    rhs.setModeString(trainName);
-    rhs.setName(trainNumber);
-    return KPublicTransport::Line::isSame(lhs, rhs);
-}
-
-static bool isDepartureForReservation(const QVariant &res, const KPublicTransport::Stopover &dep)
-{
-    const auto lineData = ReservationHelper::lineNameAndNumber(res);
-    return PublicTransport::isSameMode(res, dep.route().line().mode())
-        && SortUtil::startDateTime(res) == dep.scheduledDepartureTime()
-        && isSameLine(dep.route().line(), lineData.first, lineData.second);
-}
-
-static bool isArrivalForReservation(const QVariant &res, const KPublicTransport::Stopover &arr)
-{
-    const auto lineData = ReservationHelper::lineNameAndNumber(res);
-    return PublicTransport::isSameMode(res, arr.route().line().mode())
-        && SortUtil::endDateTime(res) == arr.scheduledArrivalTime()
-        && isSameLine(arr.route().line(), lineData.first, lineData.second);
-}
-
-static bool isJourneyForReservation(const QVariant &res, const KPublicTransport::JourneySection &journey)
-{
-    const auto lineData = ReservationHelper::lineNameAndNumber(res);
-    return PublicTransport::isSameMode(res, journey.route().line().mode())
-        && SortUtil::startDateTime(res) == journey.scheduledDepartureTime()
-        && isSameLine(journey.route().line(), lineData.first, lineData.second);
-}
-
 void LiveDataManager::checkReservation(const QVariant &res, const QString& resId)
 {
     using namespace KPublicTransport;
@@ -235,7 +205,7 @@ void LiveDataManager::stopoverQueryFinished(std::vector<KPublicTransport::Stopov
     const auto res = m_resMgr->reservation(resId);
     for (const auto &stop : result) {
         qCDebug(Log) << "Got stopover information:" << stop.route().line().name() << stop.scheduledDepartureTime();
-        if (type == LiveData::Arrival ? isArrivalForReservation(res, stop) : isDepartureForReservation(res, stop)) {
+        if (type == LiveData::Arrival ? PublicTransportMatcher::isArrivalForReservation(res, stop) : PublicTransportMatcher::isDepartureForReservation(res, stop)) {
             qCDebug(Log) << "Found stopover information:" << stop.route().line().name() << stop.expectedPlatform() << stop.expectedDepartureTime();
             updateStopoverData(stop, type, resId, res);
             return;
@@ -263,7 +233,7 @@ void LiveDataManager::journeyQueryFinished(KPublicTransport::JourneyReply *reply
         const auto it = std::find_if(journey.sections().begin(), journey.sections().end(), [](const auto &sec) { return sec.mode() == JourneySection::PublicTransport; });
         assert(it != journey.sections().end());
         qCDebug(Log) << "Got journey information:" << (*it).route().line().name() << (*it).scheduledDepartureTime();
-        if (isJourneyForReservation(res, (*it))) {
+        if (PublicTransportMatcher::isJourneyForReservation(res, (*it))) {
             qCDebug(Log) << "Found journey information:" << (*it).route().line().name() << (*it).expectedDeparturePlatform() << (*it).expectedDepartureTime();
             updateJourneyData((*it), resId, res);
             return;
@@ -405,6 +375,11 @@ bool LiveDataManager::hasDeparted(const QString &resId, const QVariant &res) con
 
 bool LiveDataManager::hasArrived(const QString &resId, const QVariant &res) const
 {
+    const auto n = now();
+    // avoid loading live data for everything on startup
+    if (SortUtil::endDateTime(res).addDays(1) < n) {
+        return true;
+    }
     return arrivalTime(resId, res) < now();
 }
 
@@ -479,20 +454,20 @@ void LiveDataManager::batchChanged(const QString &resId)
     const auto res = m_resMgr->reservation(resId);
     const auto dataIt = m_data.find(resId);
     if (dataIt != m_data.end()) {
-        if ((*dataIt).departureTimestamp.isValid() && !isDepartureForReservation(res, (*dataIt).departure)) {
+        if ((*dataIt).departureTimestamp.isValid() && !PublicTransportMatcher::isDepartureForReservation(res, (*dataIt).departure)) {
             (*dataIt).departure = {};
             (*dataIt).departureTimestamp = {};
             (*dataIt).store(resId, LiveData::Departure);
             Q_EMIT departureUpdated(resId);
         }
-        if ((*dataIt).arrivalTimestamp.isValid() && !isArrivalForReservation(res, (*dataIt).arrival)) {
+        if ((*dataIt).arrivalTimestamp.isValid() && !PublicTransportMatcher::isArrivalForReservation(res, (*dataIt).arrival)) {
             (*dataIt).arrival = {};
             (*dataIt).arrivalTimestamp = {};
             (*dataIt).store(resId, LiveData::Arrival);
             Q_EMIT arrivalUpdated(resId);
         }
 
-        if ((*dataIt).journeyTimestamp.isValid() && !isJourneyForReservation(res, (*dataIt).journey)) {
+        if ((*dataIt).journeyTimestamp.isValid() && !PublicTransportMatcher::isJourneyForReservation(res, (*dataIt).journey)) {
             (*dataIt).journey = {};
             (*dataIt).journeyTimestamp = {};
             (*dataIt).store(resId, LiveData::Journey);
@@ -651,12 +626,35 @@ QDateTime LiveDataManager::lastDeparturePollTime(const QString &batchId, const Q
 
 void LiveDataManager::pkPassUpdated(const QString &passId, const QStringList &changes)
 {
-    Q_UNUSED(passId)
-    // ### to provide more context, we need to have a passId -> batchId map here eventually
-
-    if (!changes.isEmpty()) {
-        KNotification::event(KNotification::Notification, i18n("Itinerary change"), changes.join(QLatin1Char('\n')), QLatin1String("clock"));
+    if (changes.isEmpty()) {
+        return;
     }
+
+    QVariant passRes;
+
+    // Find relevant reservation for the given passId.
+    for (const QString &resId : std::as_const(m_reservations)) {
+        const auto res = m_resMgr->reservation(resId);
+        const auto resPassId = PkPassManager::passId(res);
+        if (resPassId == passId) {
+            passRes = res;
+            break;
+        }
+    }
+
+    QString text = changes.join(QLatin1Char('\n'));
+
+    if (JsonLd::isA<FlightReservation>(passRes)) {
+        const auto flight = passRes.value<FlightReservation>().reservationFor().value<Flight>();
+
+        text.prepend(QLatin1Char('\n'));
+        text.prepend(i18n("Flight %1 to %2:",
+                          // TODO add formatter util for this.
+                          flight.airline().iataCode() + QLatin1Char(' ') + flight.flightNumber(),
+                          LocationUtil::name(LocationUtil::arrivalLocation(passRes))));
+    }
+
+    KNotification::event(KNotification::Notification, i18n("Itinerary change"), text, QLatin1String("clock"));
 }
 
 KPublicTransport::Manager* LiveDataManager::publicTransportManager() const
