@@ -11,6 +11,7 @@
 #include "applicationcontroller.h"
 #include "calendarimportmodel.h"
 #include "countrysubdivisionmodel.h"
+#include "clipboard.h"
 #include "developmentmodecontroller.h"
 #include "documentmanager.h"
 #include "documentsmodel.h"
@@ -24,11 +25,12 @@
 #include "locationinformation.h"
 #include "locationinformationdelegatecontroller.h"
 #include "mapdownloadmanager.h"
+#include "matrixcontroller.h"
 #include "navigationcontroller.h"
 #include "notificationconfigcontroller.h"
+#include "onlineticketimporter.h"
 #include "passmanager.h"
 #include "pkpassmanager.h"
-#include "timelinemodel.h"
 #include "permissionmanager.h"
 #include "pkpassimageprovider.h"
 #include "publictransport.h"
@@ -38,6 +40,7 @@
 #include "statisticstimerangemodel.h"
 #include "tickettokenmodel.h"
 #include "timelinedelegatecontroller.h"
+#include "timelinemodel.h"
 #include "timelinesectiondelegatecontroller.h"
 #include "transferdelegatecontroller.h"
 #include "transfermanager.h"
@@ -47,6 +50,11 @@
 #include "util.h"
 #include "weatherforecastmodel.h"
 
+#if HAVE_MATRIX
+#include "matrix/matrixroomsmodel.h"
+#include "matrix/matrixbeacon.h"
+#endif
+
 #include <weatherforecastmanager.h>
 
 #include <calendarlistmodel.h>
@@ -92,6 +100,15 @@
 #include <QIcon>
 #include <QWindow>
 
+#if !HAVE_MATRIX
+class MatrixBeaconStub : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(QVariant connection MEMBER m_connection)
+    QVariant m_connection;
+};
+#endif
+
 void registerKContactsTypes()
 {
     // ### this should move into a real QML plugin for KContacts
@@ -138,6 +155,7 @@ void registerApplicationTypes()
     qRegisterMetaType<TripGroupManager*>();
     qRegisterMetaType<WeatherForecast>();
     qRegisterMetaType<Permission::Permission>();
+    qRegisterMetaType<HealthCertificateManager*>();
 
     qmlRegisterUncreatableType<LocationInformation>("org.kde.itinerary", 1, 0, "LocationInformation", {});
     qmlRegisterUncreatableType<StatisticsItem>("org.kde.itinerary", 1, 0, "StatisticsItem", {});
@@ -146,6 +164,9 @@ void registerApplicationTypes()
     qmlRegisterUncreatableType<Transfer>("org.kde.itinerary", 1, 0, "Transfer", {});
 
     qmlRegisterUncreatableMetaObject(Permission::staticMetaObject, "org.kde.itinerary", 1, 0, "Permission", {});
+#if HAVE_MATRIX
+    qmlRegisterUncreatableMetaObject(MatrixRoomsModel::staticMetaObject, "org.kde.itinerary", 1, 0, "MatrixRoomsModel", {});
+#endif
 
     qmlRegisterType<CalendarImportModel>("org.kde.itinerary", 1, 0, "CalendarImportModel");
     qmlRegisterType<CountrySubdivisionModel>("org.kde.itinerary", 1, 0, "CountrySubdivisionModel");
@@ -159,6 +180,20 @@ void registerApplicationTypes()
     qmlRegisterType<TimelineSectionDelegateController>("org.kde.itinerary", 1, 0, "TimelineSectionDelegateController");
     qmlRegisterType<TransferDelegateController>("org.kde.itinerary", 1, 0, "TransferDelegateController");
     qmlRegisterType<WeatherForecastModel>("org.kde.itinerary", 1, 0, "WeatherForecastModel");
+#if HAVE_MATRIX
+    qmlRegisterType<MatrixBeacon>("org.kde.itinerary", 1, 0, "MatrixBeacon");
+#else
+    qmlRegisterType<MatrixBeaconStub>("org.kde.itinerary", 1, 0, "MatrixBeacon");
+#endif
+    qmlRegisterType<OnlineTicketImporter>("org.kde.itinerary", 1, 0, "OnlineTicketImporter");
+
+    qmlRegisterSingletonType("org.kde.itinerary", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
+        return engine->toScriptValue(KAboutData::applicationData());
+    });
+
+    qmlRegisterSingletonType<Clipboard>("org.kde.itinerary", 1, 0, "Clipboard", [](QQmlEngine *, QJSEngine *) -> QObject * {
+        return new Clipboard;
+    });
 }
 
 // for registering QML singletons only
@@ -175,8 +210,8 @@ static TimelineModel *s_timelineModel = nullptr;
 static TripGroupInfoProvider s_tripGroupInfoProvider;
 static TripGroupProxyModel *s_tripGroupProxyModel = nullptr;
 static MapDownloadManager *s_mapDownloadManager = nullptr;
-static HealthCertificateManager *s_healthCertificateManager = nullptr;
 static PassManager *s_passManager = nullptr;
+static MatrixController *s_matrixController = nullptr;
 
 #define REGISTER_SINGLETON_INSTANCE(Class, Instance) \
     qmlRegisterSingletonInstance<Class>("org.kde.itinerary", 1, 0, #Class, Instance);
@@ -206,8 +241,8 @@ void registerApplicationSingletons()
     REGISTER_SINGLETON_INSTANCE(TimelineModel, s_timelineModel)
     REGISTER_SINGLETON_INSTANCE(TripGroupProxyModel, s_tripGroupProxyModel)
     REGISTER_SINGLETON_INSTANCE(MapDownloadManager, s_mapDownloadManager)
-    REGISTER_SINGLETON_INSTANCE(HealthCertificateManager, s_healthCertificateManager)
     REGISTER_SINGLETON_INSTANCE(PassManager, s_passManager)
+    REGISTER_SINGLETON_INSTANCE(MatrixController, s_matrixController);
 
     REGISTER_SINGLETON_GADGET_INSTANCE(TripGroupInfoProvider, s_tripGroupInfoProvider)
 
@@ -278,6 +313,7 @@ int main(int argc, char **argv)
     aboutData.setHomepage(QStringLiteral("https://apps.kde.org/itinerary"));
     aboutData.setShortDescription(i18n("Digital Travel Assistant"));
     aboutData.setCopyrightStatement(i18n("Copyright © 2018-%1 The KDE Community", QStringLiteral("2022")));
+    aboutData.setDesktopFileName(QStringLiteral("org.kde.itinerary"));
     KAboutData::setApplicationData(aboutData);
 
     QCommandLineParser parser;
@@ -285,6 +321,8 @@ int main(int argc, char **argv)
     parser.addOption(isTemporaryOpt);
     QCommandLineOption pageOpt(QStringLiteral("page"), i18n("Open Itinerary on the given page"), QStringLiteral("page"));
     parser.addOption(pageOpt);
+    QCommandLineOption selfTestOpt(QStringLiteral("self-test"), QStringLiteral("internal, for automated testing"));
+    parser.addOption(selfTestOpt);
     aboutData.setupCommandLine(&parser);
     parser.addPositionalArgument(QStringLiteral("file"), i18n("PkPass or JSON-LD file to import."));
     parser.process(app);
@@ -365,13 +403,13 @@ int main(int argc, char **argv)
     QObject::connect(s_settings, &Settings::preloadMapDataChanged, &mapDownloadMgr, &MapDownloadManager::setAutomaticDownloadEnabled);
     s_mapDownloadManager = &mapDownloadMgr;
 
-    HealthCertificateManager healthCertificateMgr;
-    s_healthCertificateManager = &healthCertificateMgr;
-
     KItinerary::JsonLdDocument::registerType<GenericPkPass>();
     PassManager passMgr;
     s_passManager = &passMgr;
 
+    MatrixController matrixController;
+    s_matrixController = &matrixController;
+
     ApplicationController appController;
     appController.setReservationManager(&resMgr);
     appController.setPkPassManager(&pkPassMgr);
@@ -381,7 +419,6 @@ int main(int argc, char **argv)
     appController.setLiveDataManager(&liveDataMgr);
     appController.setTripGroupManager(&tripGroupMgr);
     appController.setPassManager(&passMgr);
-    appController.setHealthCertificateManager(&healthCertificateMgr);
 #ifndef Q_OS_ANDROID
     QObject::connect(&service, &KDBusService::activateRequested, [&](const QStringList &args, const QString &workingDir) {
         qCDebug(Log) << "remote activation" << args << workingDir;
@@ -412,6 +449,11 @@ int main(int argc, char **argv)
     engine.rootContext()->setContextObject(l10nContext);
     engine.load(QStringLiteral("qrc:/main.qml"));
 
+    // Exit on QML load error.
+    if (engine.rootObjects().isEmpty()) {
+        return 1;
+    }
+
     handleCommandLineArguments(&appController, parser.positionalArguments(), parser.isSet(isTemporaryOpt), parser.value(pageOpt));
 
 #ifdef Q_OS_ANDROID
@@ -419,5 +461,11 @@ int main(int argc, char **argv)
     appController.importFromIntent(Activity::getIntent());
 #endif
 
+    if (parser.isSet(selfTestOpt)) {
+        QTimer::singleShot(std::chrono::milliseconds(250), &app, &QCoreApplication::quit);
+    }
+
     return app.exec();
 }
+
+#include "main.moc"
